package com.dragon.tj.portal.component.exception.base;

import com.dragon.tj.portal.common.base.R;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * @author huey China.
 * @Description : 异常处理器
 * @Date Created in 2023/6/12 16:13
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {
    private Logger logger = LoggerFactory.getLogger(getClass());

    /**
     * 请求方式不支持
     */
    @ExceptionHandler({HttpRequestMethodNotSupportedException.class})
    @ResponseStatus(code = HttpStatus.METHOD_NOT_ALLOWED)
    public R handleException(HttpRequestMethodNotSupportedException e) {
        logger.error(e.getMessage(), e);
        return R.failed("不支持' " + e.getMethod() + "'请求");
    }

    @ExceptionHandler(Exception.class)
    public R handleException(Exception e) throws Exception {
        logger.error(e.getMessage(), e);
        return R.failed("服务器错误，请联系管理员");
    }

    @ExceptionHandler(MethodArgumentNotValidException.class)
    public R handleValidException(MethodArgumentNotValidException e) {
        //日志记录错误信息
        String defaultMessage = e.getBindingResult().getFieldError().getDefaultMessage();
        log.error("=================校验器触发错误================" + defaultMessage);
        //将错误信息返回给前台
        return R.failed(defaultMessage);
    }
}
