
package com.dragon.tj.portal.component.message;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
* @author huey China.
* @Description : 生产
* @Date Created in 2023/6/15 15:42
*/
@Component
@Slf4j
public class MessageProducer {

    @Autowired
    private KafkaTemplate<String, Object> kafkaTemplate;

    public boolean send(String topic, String msg) {
        if (!StrUtil.isAllNotEmpty(topic, msg)) {
            log.error("topic={} or/and msg={} is empty. Will not be sent to Topic.", topic, msg);
            return false;
        }
        log.info("msg will be sent to topic:{}, msg={}", topic, msg);

        try {
            kafkaTemplate.send(topic, msg).get();
            log.info("success send msg to topic:{} message:{}", topic, msg);
        } catch (Exception e) {
            log.error("send msg fail, topic:{} message: {}, err msg:{}", topic, msg, e.getMessage());
            return false;
        }
        return true;
    }
}
